#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Tunnel API Library"""
import functools

from osc_lib.api import api
from osc_lib import exceptions as osc_exc

from tunnelclient.api import constants as const
from tunnelclient.api import exceptions

TunnelClientException = exceptions.TunnelClientException


def correct_return_codes(func):
    _status_dict = {400: 'Bad Request', 401: 'Unauthorized',
                    403: 'Forbidden', 404: 'Not found',
                    409: 'Conflict', 413: 'Over Limit',
                    501: 'Not Implemented', 503: 'Service Unavailable'}

    def wrapper(*args, **kwargs):
        try:
            response = func(*args, **kwargs)
        except Exception as e:
            code = None
            message = 'Unknown Error'
            request_id = "n/a"
            if hasattr(e, 'request_id'):
                request_id = e.request_id
            if hasattr(e, 'response'):
                code = e.response.status_code
                try:
                    message = e.response.json().get(
                        'faultstring',
                        _status_dict.get(code, message))
                except Exception:
                    message = _status_dict.get(code, message)
            elif (isinstance(e, osc_exc.ClientException) and
                    e.code != e.http_status):
                # cover https://review.opendev.org/675328 case
                code = e.http_status
                message = e.code
            else:
                raise

            raise TunnelClientException(
                code=code,
                message=message,
                request_id=request_id) from e
        return response
    return wrapper


class TunnelAPI(api.BaseAPI):
    """Tunnel API"""

    _endpoint_suffix = '/v2.0'

    def __init__(self, endpoint=None, **kwargs):
        super().__init__(endpoint=endpoint, **kwargs)
        self.endpoint = self.endpoint.rstrip('/')
        self._build_url()

        # Make sure we are always requesting JSON responses
        JSON_HEADER = {'Accept': 'application/json'}
        self._create = functools.partial(self.create, headers=JSON_HEADER)
        self._delete = functools.partial(self.delete, headers=JSON_HEADER)
        self._find = functools.partial(self.find, headers=JSON_HEADER)
        self._list = functools.partial(self.list, headers=JSON_HEADER)

    def _build_url(self):
        if not self.endpoint.endswith(self._endpoint_suffix):
            self.endpoint += self._endpoint_suffix

    def vpn_server_list(self, **params):
        """List all vpn servers

        :param params:
            Parameters to filter on
        :return:
            List of vpn servers
        """
        url = const.BASE_VPNSERVER_URL
        response = self._list(url, **params)

        return response

    @correct_return_codes
    def vpn_server_show(self, lb_id):
        """Show a vpn server

        :param string lb_id:
            ID of the vpn server to show
        :return:
            A dict of the specified vpn server's settings
        """
        response = self._find(path=const.BASE_VPNSERVER_URL, value=lb_id)

        return response

    @correct_return_codes
    def vpn_server_create(self, **params):
        """Create a vpn server

        :param params:
            Paramaters to create the vpn server with (expects json=)
        :return:
            A dict of the created vpn server's settings
        """
        url = const.BASE_VPNSERVER_URL
        response = self._create(url, **params)

        return response

    @correct_return_codes
    def vpn_server_delete(self, lb_id, **params):
        """Delete a vpn server

        :param string lb_id:
            The ID of the vpn server to delete
        :param params:
            A dict of url parameters
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_LB_URL.format(uuid=lb_id)
        response = self._delete(url, params=params)

        return response

    @correct_return_codes
    def vpn_server_set(self, lb_id, **params):
        """Update a vpn server's settings

        :param string lb_id:
            The ID of the vpn server to update
        :param params:
            A dict of arguments to update a vpnserver
        :return:
            Response Code from API
        """
        url = const.BASE_SINGLE_LB_URL.format(uuid=lb_id)
        response = self._create(url, method='PUT', **params)

        return response

    def vpn_server_stats_show(self, lb_id, **kwargs):
        """Shows the current statistics for a vpn server.

        :param string lb_id:
            ID of the vpn server
        :return:
            A dict of the specified vpn server's statistics
        """
        url = const.BASE_LB_STATS_URL.format(uuid=lb_id)
        response = self._list(url, **kwargs)

        return response

    def vpn_server_status_show(self, lb_id, **kwargs):
        """Display vpn server status tree in json format.

        :param string lb_id:
            ID of the vpn server
        :return:
            A dict of the specified vpn server's status
        """
        url = const.BASE_VPNSERVER_STATUS_URL.format(uuid=lb_id)
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def vpn_server_failover(self, lb_id):
        """Trigger vpn server failover

        :param string lb_id:
            ID of the vpn server to failover
        :return:
            Response Code from the API
        """
        url = const.BASE_VPNSERVER_FAILOVER_URL.format(uuid=lb_id)
        response = self._create(url, method='PUT')

        return response

    def listener_list(self, **kwargs):
        """List all listeners

        :param kwargs:
            Parameters to filter on
        :return:
            List of listeners
        """
        url = const.BASE_LISTENER_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def listener_show(self, listener_id):
        """Show a listener

        :param string listener_id:
            ID of the listener to show
        :return:
            A dict of the specified listener's settings
        """
        response = self._find(path=const.BASE_LISTENER_URL, value=listener_id)

        return response

    @correct_return_codes
    def listener_create(self, **kwargs):
        """Create a listener

        :param kwargs:
            Parameters to create a listener with (expects json=)
        :return:
            A dict of the created listener's settings
        """
        url = const.BASE_LISTENER_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def listener_delete(self, listener_id):
        """Delete a listener

        :param stirng listener_id:
            ID of of listener to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_LISTENER_URL.format(uuid=listener_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def listener_set(self, listener_id, **kwargs):
        """Update a listener's settings

        :param string listener_id:
            ID of the listener to update
        :param kwargs:
            A dict of arguments to update a listener
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_LISTENER_URL.format(uuid=listener_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def listener_stats_show(self, listener_id, **kwargs):
        """Shows the current statistics for a listener

        :param string listener_id:
            ID of the listener
        :return:
            A dict of the specified listener's statistics
        """
        url = const.BASE_LISTENER_STATS_URL.format(uuid=listener_id)
        response = self._list(url, **kwargs)

        return response

    def user_list(self, **kwargs):
        """List all users

        :param kwargs:
            Parameters to filter on
        :return:
            List of users
        """
        url = const.BASE_USER_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def user_show(self, user_id):
        """Show a user

        :param string user_id:
            ID of the user to show
        :return:
            A dict of the specified user's settings
        """
        response = self._find(path=const.BASE_USER_URL, value=user_id)

        return response

    @correct_return_codes
    def user_create(self, **kwargs):
        """Create a user

        :param kwargs:
            Parameters to create a user with (expects json=)
        :return:
            A dict of the created user's settings
        """
        url = const.BASE_USER_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def user_delete(self, user_id):
        """Delete a user

        :param stirng user_id:
            ID of of user to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_USER_URL.format(uuid=user_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def user_set(self, user_id, **kwargs):
        """Update a user's settings

        :param string user_id:
            ID of the user to update
        :param kwargs:
            A dict of arguments to update a user
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_USER_URL.format(uuid=user_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def user_stats_show(self, user_id, **kwargs):
        """Shows the current statistics for a user

        :param string user_id:
            ID of the user
        :return:
            A dict of the specified user's statistics
        """
        url = const.BASE_USER_STATS_URL.format(uuid=user_id)
        response = self._list(url, **kwargs)

        return response

# certificate
    def certificate_list(self, **kwargs):
        """List all certificates

        :param kwargs:
            Parameters to filter on
        :return:
            List of certificates
        """
        url = const.BASE_CERTIFICATE_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def certificate_show(self, certificate_id):
        """Show a certificate

        :param string certificate_id:
            ID of the certificate to show
        :return:
            A dict of the specified certificate's settings
        """
        response = self._find(path=const.BASE_CERTIFICATE_URL, value=certificate_id)

        return response

    @correct_return_codes
    def certificate_create(self, **kwargs):
        """Create a certificate

        :param kwargs:
            Parameters to create a certificate with (expects json=)
        :return:
            A dict of the created certificate's settings
        """
        url = const.BASE_CERTIFICATE_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def certificate_delete(self, certificate_id):
        """Delete a certificate

        :param stirng certificate_id:
            ID of of certificate to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_CERTIFICATE_URL.format(uuid=certificate_id)
        response = self._delete(url)

        return response


    def resource_list(self, **kwargs):
        """List all resources

        :param kwargs:
            Parameters to filter on
        :return:
            List of resources
        """
        url = const.BASE_RESOURCE_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def resource_show(self, resource_id):
        """Show a resource

        :param string resource_id:
            ID of the resource to show
        :return:
            A dict of the specified resource's settings
        """
        response = self._find(path=const.BASE_RESOURCE_URL, value=resource_id)

        return response

    @correct_return_codes
    def resource_create(self, **kwargs):
        """Create a resource

        :param kwargs:
            Parameters to create a resource with (expects json=)
        :return:
            A dict of the created resource's settings
        """
        url = const.BASE_RESOURCE_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def resource_delete(self, resource_id):
        """Delete a resource

        :param stirng resource_id:
            ID of of resource to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_RESOURCE_URL.format(uuid=resource_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def resource_set(self, resource_id, **kwargs):
        """Update a resource's settings

        :param string resource_id:
            ID of the resource to update
        :param kwargs:
            A dict of arguments to update a resource
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_RESOURCE_URL.format(uuid=resource_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def resource_stats_show(self, resource_id, **kwargs):
        """Shows the current statistics for a resource

        :param string resource_id:
            ID of the resource
        :return:
            A dict of the specified resource's statistics
        """
        url = const.BASE_RESOURCE_STATS_URL.format(uuid=resource_id)
        response = self._list(url, **kwargs)

        return response

    def secgroup_list(self, **kwargs):
        """List all secgroups

        :param kwargs:
            Parameters to filter on
        :return:
            List of secgroups
        """
        url = const.BASE_SECGROUP_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def secgroup_show(self, secgroup_id):
        """Show a secgroup

        :param string secgroup_id:
            ID of the secgroup to show
        :return:
            A dict of the specified secgroup's settings
        """
        response = self._find(path=const.BASE_SECGROUP_URL, value=secgroup_id)

        return response

    @correct_return_codes
    def secgroup_create(self, **kwargs):
        """Create a secgroup

        :param kwargs:
            Parameters to create a secgroup with (expects json=)
        :return:
            A dict of the created secgroup's settings
        """
        url = const.BASE_SECGROUP_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def secgroup_delete(self, secgroup_id):
        """Delete a secgroup

        :param stirng secgroup_id:
            ID of of secgroup to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_SECGROUP_URL.format(uuid=secgroup_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def secgroup_set(self, secgroup_id, **kwargs):
        """Update a secgroup's settings

        :param string secgroup_id:
            ID of the secgroup to update
        :param kwargs:
            A dict of arguments to update a secgroup
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_SECGROUP_URL.format(uuid=secgroup_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def secgroup_stats_show(self, secgroup_id, **kwargs):
        """Shows the current statistics for a secgroup

        :param string secgroup_id:
            ID of the secgroup
        :return:
            A dict of the specified secgroup's statistics
        """
        url = const.BASE_SECGROUP_STATS_URL.format(uuid=secgroup_id)
        response = self._list(url, **kwargs)

        return response

    def pool_list(self, **kwargs):
        """List all pools

        :param kwargs:
            Parameters to filter on
        :return:
            List of pools
        """
        url = const.BASE_POOL_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def pool_create(self, **kwargs):
        """Create a pool

        :param kwargs:
            Parameters to create a pool with (expects json=)
        :return:
            A dict of the created pool's settings
        """
        url = const.BASE_POOL_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def pool_delete(self, pool_id):
        """Delete a pool

        :param string pool_id:
            ID of of pool to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_POOL_URL.format(pool_id=pool_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def pool_show(self, pool_id):
        """Show a pool's settings

        :param string pool_id:
            ID of the pool to show
        :return:
            Dict of the specified pool's settings
        """
        response = self._find(path=const.BASE_POOL_URL, value=pool_id)

        return response

    @correct_return_codes
    def pool_set(self, pool_id, **kwargs):
        """Update a pool's settings

        :param pool_id:
            ID of the pool to update
        :param kwargs:
            A dict of arguments to update a pool
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_POOL_URL.format(pool_id=pool_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def member_list(self, pool_id, **kwargs):
        """Lists the member from a given pool id

        :param pool_id:
            ID of the pool
        :param kwargs:
            A dict of filter arguments
        :return:
            Response list members
        """
        url = const.BASE_MEMBER_URL.format(pool_id=pool_id)
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def member_show(self, pool_id, member_id):
        """Showing a member details of a pool

        :param pool_id:
            ID of pool the member is added
        :param member_id:
            ID of the member
        :param kwargs:
            A dict of arguments
        :return:
            Response of member
        """
        url = const.BASE_MEMBER_URL.format(pool_id=pool_id)
        response = self._find(path=url, value=member_id)

        return response

    @correct_return_codes
    def member_create(self, pool_id, **kwargs):
        """Creating a member for the given pool id

        :param pool_id:
            ID of pool to which member is added
        :param kwargs:
            A Dict of arguments
        :return:
            A member details on successful creation
        """
        url = const.BASE_MEMBER_URL.format(pool_id=pool_id)
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def member_delete(self, pool_id, member_id):
        """Removing a member from a pool and mark that member as deleted

        :param pool_id:
            ID of the pool
        :param member_id:
            ID of the member to be deleted
        :return:
            Response code from the API
        """
        url = const.BASE_SINGLE_MEMBER_URL.format(pool_id=pool_id,
                                                  member_id=member_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def member_set(self, pool_id, member_id, **kwargs):
        """Updating a member settings

        :param pool_id:
            ID of the pool
        :param member_id:
            ID of the member to be updated
        :param kwargs:
            A dict of the values of member to be updated
        :return:
            Response code from the API
        """
        url = const.BASE_SINGLE_MEMBER_URL.format(pool_id=pool_id,
                                                  member_id=member_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def l7policy_list(self, **kwargs):
        """List all l7policies

        :param kwargs:
            Parameters to filter on
        :return:
            List of l7policies
        """
        url = const.BASE_L7POLICY_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def l7policy_create(self, **kwargs):
        """Create a l7policy

        :param kwargs:
            Parameters to create a l7policy with (expects json=)
        :return:
            A dict of the created l7policy's settings
        """
        url = const.BASE_L7POLICY_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def l7policy_delete(self, l7policy_id):
        """Delete a l7policy

        :param string l7policy_id:
            ID of of l7policy to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_L7POLICY_URL.format(policy_uuid=l7policy_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def l7policy_show(self, l7policy_id):
        """Show a l7policy's settings

        :param string l7policy_id:
            ID of the l7policy to show
        :return:
            Dict of the specified l7policy's settings
        """
        response = self._find(path=const.BASE_L7POLICY_URL, value=l7policy_id)

        return response

    @correct_return_codes
    def l7policy_set(self, l7policy_id, **kwargs):
        """Update a l7policy's settings

        :param l7policy_id:
            ID of the l7policy to update
        :param kwargs:
            A dict of arguments to update a l7policy
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_L7POLICY_URL.format(policy_uuid=l7policy_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def l7rule_list(self, l7policy_id, **kwargs):
        """List all l7rules for a l7policy

        :param kwargs:
            Parameters to filter on
        :return:
            List of l7rules
        """
        url = const.BASE_L7RULE_URL.format(policy_uuid=l7policy_id)
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def l7rule_create(self, l7policy_id, **kwargs):
        """Create a l7rule

        :param string l7policy_id:
            The l7policy to create the l7rule for
        :param kwargs:
            Parameters to create a l7rule with (expects json=)
        :return:
            A dict of the created l7rule's settings
        """
        url = const.BASE_L7RULE_URL.format(policy_uuid=l7policy_id)
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def l7rule_delete(self, l7rule_id, l7policy_id):
        """Delete a l7rule

        :param string l7rule_id:
            ID of of l7rule to delete
        :param string l7policy_id:
            ID of the l7policy for this l7rule
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_L7RULE_URL.format(rule_uuid=l7rule_id,
                                                  policy_uuid=l7policy_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def l7rule_show(self, l7rule_id, l7policy_id):
        """Show a l7rule's settings

        :param string l7rule_id:
            ID of the l7rule to show
        :param string l7policy_id:
            ID of the l7policy for this l7rule
        :return:
            Dict of the specified l7rule's settings
        """
        url = const.BASE_L7RULE_URL.format(policy_uuid=l7policy_id)
        response = self._find(path=url, value=l7rule_id)

        return response

    @correct_return_codes
    def l7rule_set(self, l7rule_id, l7policy_id, **kwargs):
        """Update a l7rule's settings

        :param l7rule_id:
            ID of the l7rule to update
        :param string l7policy_id:
            ID of the l7policy for this l7rule
        :param kwargs:
            A dict of arguments to update a l7rule
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_L7RULE_URL.format(rule_uuid=l7rule_id,
                                                  policy_uuid=l7policy_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def health_monitor_list(self, **kwargs):
        """List all health monitors

        :param kwargs:
            Parameters to filter on
        :return:
            A dict containing a list of health monitors
        """
        url = const.BASE_HEALTH_MONITOR_URL
        response = self._list(url, **kwargs)

        return response

    @correct_return_codes
    def health_monitor_create(self, **kwargs):
        """Create a health monitor

        :param kwargs:
            Parameters to create a health monitor with (expects json=)
        :return:
            A dict of the created health monitor's settings
        """
        url = const.BASE_HEALTH_MONITOR_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def health_monitor_delete(self, health_monitor_id):
        """Delete a health_monitor

        :param string health_monitor_id:
            ID of of health monitor to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_HEALTH_MONITOR_URL.format(
            uuid=health_monitor_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def health_monitor_show(self, health_monitor_id):
        """Show a health monitor's settings

        :param string health_monitor_id:
            ID of the health monitor to show
        :return:
            Dict of the specified health monitor's settings
        """
        url = const.BASE_HEALTH_MONITOR_URL
        response = self._find(path=url, value=health_monitor_id)

        return response

    @correct_return_codes
    def health_monitor_set(self, health_monitor_id, **kwargs):
        """Update a health monitor's settings

        :param health_monitor_id:
            ID of the health monitor to update
        :param kwargs:
            A dict of arguments to update a health monitor
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_HEALTH_MONITOR_URL.format(
            uuid=health_monitor_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    def quota_list(self, **params):
        """List all quotas

        :param params:
            Parameters to filter on (not implemented)
        :return:
            A ``dict`` representing a list of quotas for the project
        """
        url = const.BASE_QUOTA_URL
        response = self._list(url, **params)

        return response

    @correct_return_codes
    def quota_show(self, project_id):
        """Show a quota

        :param string project_id:
            ID of the project to show
        :return:
            A ``dict`` representing the quota for the project
        """
        response = self._find(path=const.BASE_QUOTA_URL, value=project_id)

        return response

    @correct_return_codes
    def quota_reset(self, project_id):
        """Reset a quota

        :param string project_id:
            The ID of the project to reset quotas
        :return:
            ``None``
        """
        url = const.BASE_SINGLE_QUOTA_URL.format(uuid=project_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def quota_set(self, project_id, **params):
        """Update a quota's settings

        :param string project_id:
            The ID of the project to update
        :param params:
            A ``dict`` of arguments to update project quota
        :return:
            A ``dict`` representing the updated quota
        """
        url = const.BASE_SINGLE_QUOTA_URL.format(uuid=project_id)
        response = self._create(url, method='PUT', **params)

        return response

    def quota_defaults_show(self):
        """Show quota defaults

        :return:
            A ``dict`` representing a list of quota defaults
        """
        url = const.BASE_QUOTA_DEFAULT_URL
        response = self._list(url)

        return response

    @correct_return_codes
    def amphora_show(self, amphora_id):
        """Show an amphora

        :param string amphora_id:
            ID of the amphora to show
        :return:
            A ``dict`` of the specified amphora's attributes
        """
        url = const.BASE_AMPHORA_URL
        response = self._find(path=url, value=amphora_id)

        return response

    def amphora_list(self, **kwargs):
        """List all amphorae

        :param kwargs:
            Parameters to filter on
        :return:
            A ``dict`` containing a list of amphorae
        """
        url = const.BASE_AMPHORA_URL
        response = self._list(path=url, **kwargs)

        return response

    @correct_return_codes
    def amphora_configure(self, amphora_id):
        """Update the amphora agent configuration

        :param string amphora_id:
            ID of the amphora to configure
        :return:
            Response Code from the API
        """
        url = const.BASE_AMPHORA_CONFIGURE_URL.format(uuid=amphora_id)
        response = self._create(url, method='PUT')

        return response

    @correct_return_codes
    def amphora_delete(self, amphora_id):
        """Delete an amphora

        :param string amphora_id:
            The ID of the amphora to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_AMPHORA_URL.format(uuid=amphora_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def amphora_failover(self, amphora_id):
        """Force failover an amphorae

        :param string amphora_id:
            ID of the amphora to failover
        :return:
            Response Code from the API
        """
        url = const.BASE_AMPHORA_FAILOVER_URL.format(uuid=amphora_id)
        response = self._create(url, method='PUT')

        return response

    def amphora_stats_show(self, amphora_id, **kwargs):
        """Show the current statistics for an amphora

        :param string amphora_id:
            ID of the amphora to show
        :return:
            A ``list`` of ``dict`` of the specified amphora's statistics
        """
        url = const.BASE_AMPHORA_STATS_URL.format(uuid=amphora_id)
        response = self._list(path=url, **kwargs)

        return response

    def provider_list(self):
        """List all providers

        :return:
            A ``dict`` containing a list of provider
        """
        url = const.BASE_PROVIDER_URL
        response = self._list(path=url)

        return response

    def provider_flavor_capability_list(self, provider):
        """Show the flavor capabilities of the specified provider.

        :param string provider:
            The name of the provider to show
        :return:
            A ``dict`` containing the capabilities of the provider
        """
        url = const.BASE_PROVIDER_FLAVOR_CAPABILITY_URL.format(
            provider=provider)
        response = self._list(url)

        return response

    def provider_availability_zone_capability_list(self, provider):
        """Show the availability zone capabilities of the specified provider.

        :param string provider:
            The name of the provider to show
        :return:
            A ``dict`` containing the capabilities of the provider
        """
        url = const.BASE_PROVIDER_AVAILABILITY_ZONE_CAPABILITY_URL.format(
            provider=provider)
        response = self._list(url)

        return response

    def flavor_list(self, **kwargs):
        """List all flavors

        :param kwargs:
            Parameters to filter on
        :return:
            A ``dict`` containing a list of flavor
        """
        url = const.BASE_FLAVOR_URL
        response = self._list(path=url, **kwargs)

        return response

    @correct_return_codes
    def flavor_delete(self, flavor_id):
        """Delete a flavor

        :param string flavor_id:
            ID of the flavor to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_FLAVOR_URL.format(uuid=flavor_id)
        response = self._delete(url)

        return response

    @correct_return_codes
    def flavor_create(self, **kwargs):
        """Create a flavor

        :param kwargs:
            Parameters to create a flavor with (expects json=)
        :return:
            A dict of the created flavor's settings
        """
        url = const.BASE_FLAVOR_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def flavor_set(self, flavor_id, **kwargs):
        """Update a flavor's settings

        :param string flavor_id:
            ID of the flavor to update
        :param kwargs:
            A dict of arguments to update a flavor
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_FLAVOR_URL.format(uuid=flavor_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    @correct_return_codes
    def flavor_show(self, flavor_id):
        """Show a flavor

        :param string flavor_id:
            ID of the flavor to show
        :return:
            A dict of the specified flavor's settings
        """
        response = self._find(path=const.BASE_FLAVOR_URL, value=flavor_id)

        return response

    @correct_return_codes
    def flavorprofile_create(self, **kwargs):
        """Create a flavor profile

        :param kwargs:
            Parameters to create a flavor profile with (expects json=)
        :return:
            A dict of the created flavor profile's settings
        """
        url = const.BASE_FLAVORPROFILE_URL
        response = self._create(url, **kwargs)

        return response

    def flavorprofile_list(self, **kwargs):
        """List all flavor profiles

        :param kwargs:
            Parameters to filter on
        :return:
            List of flavor profile
        """
        url = const.BASE_FLAVORPROFILE_URL
        response = self._list(url, **kwargs)

        return response

    def flavorprofile_show(self, flavorprofile_id):
        """Show a flavor profile

        :param string flavorprofile_id:
            ID of the flavor profile to show
        :return:
            A dict of the specified flavor profile's settings
        """
        response = self._find(path=const.BASE_FLAVORPROFILE_URL,
                              value=flavorprofile_id)

        return response

    @correct_return_codes
    def flavorprofile_set(self, flavorprofile_id, **kwargs):
        """Update a flavor profile's settings

        :param string flavorprofile_id:
            ID of the flavor profile to update
        :kwargs:
            A dict of arguments to update the flavor profile
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_FLAVORPROFILE_URL.format(uuid=flavorprofile_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    @correct_return_codes
    def flavorprofile_delete(self, flavorprofile_id):
        """Delete a flavor profile

        :param string flavorprofile_id:
            ID of the flavor profile to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_FLAVORPROFILE_URL.format(uuid=flavorprofile_id)
        response = self._delete(url)

        return response

    def availabilityzone_list(self, **kwargs):
        """List all availabilityzones

        :param kwargs:
            Parameters to filter on
        :return:
            A ``dict`` containing a list of availabilityzone
        """
        url = const.BASE_AVAILABILITYZONE_URL
        response = self._list(path=url, **kwargs)

        return response

    @correct_return_codes
    def availabilityzone_delete(self, availabilityzone_name):
        """Delete a availabilityzone

        :param string availabilityzone_name:
            Name of the availabilityzone to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_AVAILABILITYZONE_URL.format(
            name=availabilityzone_name)
        response = self._delete(url)

        return response

    @correct_return_codes
    def availabilityzone_create(self, **kwargs):
        """Create a availabilityzone

        :param kwargs:
            Parameters to create a availabilityzone with (expects json=)
        :return:
            A dict of the created availabilityzone's settings
        """
        url = const.BASE_AVAILABILITYZONE_URL
        response = self._create(url, **kwargs)

        return response

    @correct_return_codes
    def availabilityzone_set(self, availabilityzone_name, **kwargs):
        """Update a availabilityzone's settings

        :param string availabilityzone_name:
            Name of the availabilityzone to update
        :param kwargs:
            A dict of arguments to update a availabilityzone
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_AVAILABILITYZONE_URL.format(
            name=availabilityzone_name)
        response = self._create(url, method='PUT', **kwargs)

        return response

    @correct_return_codes
    def availabilityzone_show(self, availabilityzone_name):
        """Show a availabilityzone

        :param string availabilityzone_name:
            Name of the availabilityzone to show
        :return:
            A dict of the specified availabilityzone's settings
        """
        response = self._find(path=const.BASE_AVAILABILITYZONE_URL,
                              value=availabilityzone_name)

        return response

    @correct_return_codes
    def availabilityzoneprofile_create(self, **kwargs):
        """Create a availabilityzone profile

        :param kwargs:
            Parameters to create a availabilityzone profile with
            (expects json=)
        :return:
            A dict of the created availabilityzone profile's settings
        """
        url = const.BASE_AVAILABILITYZONEPROFILE_URL
        response = self._create(url, **kwargs)

        return response

    def availabilityzoneprofile_list(self, **kwargs):
        """List all availabilityzone profiles

        :param kwargs:
            Parameters to filter on
        :return:
            List of availabilityzone profile
        """
        url = const.BASE_AVAILABILITYZONEPROFILE_URL
        response = self._list(url, **kwargs)

        return response

    def availabilityzoneprofile_show(self, availabilityzoneprofile_id):
        """Show a availabilityzone profile

        :param string availabilityzoneprofile_id:
            ID of the availabilityzone profile to show
        :return:
            A dict of the specified availabilityzone profile's settings
        """
        response = self._find(path=const.BASE_AVAILABILITYZONEPROFILE_URL,
                              value=availabilityzoneprofile_id)

        return response

    @correct_return_codes
    def availabilityzoneprofile_set(self, availabilityzoneprofile_id,
                                    **kwargs):
        """Update a availabilityzone profile's settings

        :param string availabilityzoneprofile_id:
            ID of the availabilityzone profile to update
        :kwargs:
            A dict of arguments to update the availabilityzone profile
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_AVAILABILITYZONEPROFILE_URL.format(
            uuid=availabilityzoneprofile_id)
        response = self._create(url, method='PUT', **kwargs)

        return response

    @correct_return_codes
    def availabilityzoneprofile_delete(self, availabilityzoneprofile_id):
        """Delete a availabilityzone profile

        :param string availabilityzoneprofile_id:
            ID of the availabilityzone profile to delete
        :return:
            Response Code from the API
        """
        url = const.BASE_SINGLE_AVAILABILITYZONEPROFILE_URL.format(
            uuid=availabilityzoneprofile_id)
        response = self._delete(url)

        return response



#    Copyright 2015
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from oslotest import base

from tunnelclient.hacking import checks


class HackingTestCase(base.BaseTestCase):
    """Hacking test class.

    This class tests the hacking checks in tunnelclient.hacking.checks by
    passing strings to the check methods like the pep8/flake8 parser would. The
    parser loops over each line in the file and then passes the parameters to
    the check method. The parameter names in the check method dictate what type
    of object is passed to the check method. The parameter types are::

        logical_line: A processed line with the following modifications:
            - Multi-line statements converted to a single line.
            - Stripped left and right.
            - Contents of strings replaced with "xxx" of same length.
            - Comments removed.
        physical_line: Raw line of text from the input file.
        lines: a list of the raw lines from the input file
        tokens: the tokens that contribute to this logical line
        line_number: line number in the input file
        total_lines: number of lines in the input file
        blank_lines: blank lines before this one
        indent_char: indentation character in this file (" " or "\t")
        indent_level: indentation (with tabs expanded to multiples of 8)
        previous_indent_level: indentation on previous line
        previous_logical: previous logical line
        filename: Path of the file being run through pep8

    When running a test on a check method the return will be False/None if
    there is no violation in the sample input. If there is an error a tuple is
    returned with a position in the line, and a message. So to check the result
    just assertTrue if the check is expected to fail and assertFalse if it
    should pass.
    """

    def assertLinePasses(self, func, *args):
        with testtools.ExpectedException(StopIteration):
            next(func(*args))

    def assertLineFails(self, func, *args):
        self.assertIsInstance(next(func(*args)), tuple)

    def test_assert_true_instance(self):
        self.assertEqual(1, len(list(checks.assert_true_instance(
            "self.assertTrue(isinstance(e, "
            "exception.BuildAbortException))"))))

        self.assertEqual(0, len(list(checks.assert_true_instance(
            "self.assertTrue()"))))

    def test_assert_equal_or_not_none(self):
        self.assertEqual(1, len(list(checks.assert_equal_or_not_none(
            "self.assertEqual(A, None)"))))

        self.assertEqual(1, len(list(checks.assert_equal_or_not_none(
            "self.assertEqual(None, A)"))))

        self.assertEqual(1, len(list(checks.assert_equal_or_not_none(
            "self.assertNotEqual(A, None)"))))

        self.assertEqual(1, len(list(checks.assert_equal_or_not_none(
            "self.assertNotEqual(None, A)"))))

        self.assertEqual(0,
                         len(list(checks.assert_equal_or_not_none(
                             "self.assertIsNone()"))))

        self.assertEqual(0,
                         len(list(checks.assert_equal_or_not_none(
                             "self.assertIsNotNone()"))))

    def test_no_mutable_default_args(self):
        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "def foo (bar):"))))
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def foo (bar=[]):"))))
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def foo (bar={}):"))))

    def test_assert_equal_in(self):
        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual(a in b, True)"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual('str' in 'string', True)"))))

        self.assertEqual(0, len(list(checks.assert_equal_in(
            "self.assertEqual(any(a==1 for a in b), True)"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual(True, a in b)"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual(True, 'str' in 'string')"))))

        self.assertEqual(0, len(list(checks.assert_equal_in(
            "self.assertEqual(True, any(a==1 for a in b))"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual(a in b, False)"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual('str' in 'string', False)"))))

        self.assertEqual(0, len(list(checks.assert_equal_in(
            "self.assertEqual(any(a==1 for a in b), False)"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual(False, a in b)"))))

        self.assertEqual(1, len(list(checks.assert_equal_in(
            "self.assertEqual(False, 'str' in 'string')"))))

        self.assertEqual(0, len(list(checks.assert_equal_in(
            "self.assertEqual(False, any(a==1 for a in b))"))))

    def test_assert_equal_true_or_false(self):
        self.assertEqual(1, len(list(checks.assert_equal_true_or_false(
            "self.assertEqual(True, A)"))))

        self.assertEqual(1, len(list(checks.assert_equal_true_or_false(
            "self.assertEqual(False, A)"))))

        self.assertEqual(0, len(list(checks.assert_equal_true_or_false(
            "self.assertTrue()"))))

        self.assertEqual(0, len(list(checks.assert_equal_true_or_false(
            "self.assertFalse()"))))

    def test_no_log_warn(self):
        self.assertEqual(1, len(list(checks.no_log_warn(
            "LOG.warn()"))))

        self.assertEqual(0, len(list(checks.no_log_warn(
            "LOG.warning()"))))

    def test_no_xrange(self):
        self.assertEqual(1, len(list(checks.no_xrange(
            "xrange(45)"))))

        self.assertEqual(0, len(list(checks.no_xrange(
            "range(45)"))))

    def test_no_log_translations(self):
        for log in checks._all_log_levels:
            for hint in checks._all_hints:
                bad = 'LOG.%s(%s("Bad"))' % (log, hint)
                self.assertEqual(
                    1, len(list(checks.no_translate_logs(bad, 'f'))))
                # Catch abuses when used with a variable and not a literal
                bad = 'LOG.%s(%s(msg))' % (log, hint)
                self.assertEqual(
                    1, len(list(checks.no_translate_logs(bad, 'f'))))
                # Do not do validations in tests
                ok = 'LOG.%s(_("OK - unit tests"))' % log
                self.assertEqual(
                    0, len(list(checks.no_translate_logs(ok, 'f/tests/f'))))

    def test_check_localized_exception_messages(self):
        f = checks.check_raised_localized_exceptions
        self.assertLineFails(f, "     raise KeyError('Error text')", '')
        self.assertLineFails(f, ' raise KeyError("Error text")', '')
        self.assertLinePasses(f, ' raise KeyError(_("Error text"))', '')
        self.assertLinePasses(f, ' raise KeyError(_ERR("Error text"))', '')
        self.assertLinePasses(f, " raise KeyError(translated_msg)", '')
        self.assertLinePasses(f, '# raise KeyError("Not translated")', '')
        self.assertLinePasses(f, 'print("raise KeyError("Not '
                                 'translated")")', '')

    def test_check_localized_exception_message_skip_tests(self):
        f = checks.check_raised_localized_exceptions
        self.assertLinePasses(f, "raise KeyError('Error text')",
                              'neutron_lib/tests/unit/mytest.py')

    def test_check_no_basestring(self):
        self.assertEqual(1, len(list(checks.check_no_basestring(
            "isinstance('foo', basestring)"))))

        self.assertEqual(0, len(list(checks.check_no_basestring(
            "isinstance('foo', six.string_types)"))))

    def test_check_no_eventlet_imports(self):
        f = checks.check_no_eventlet_imports
        self.assertLinePasses(f, 'from not_eventlet import greenthread')
        self.assertLineFails(f, 'from eventlet import greenthread')
        self.assertLineFails(f, 'import eventlet')

    def test_line_continuation_no_backslash(self):
        results = list(checks.check_line_continuation_no_backslash(
            '', [(1, 'import', (2, 0), (2, 6), 'import \\\n'),
                 (1, 'os', (3, 4), (3, 6), '    os\n')]))
        self.assertEqual(1, len(results))
        self.assertEqual((2, 7), results[0][0])



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from oslo_utils import uuidutils

AMPHORA_ATTRS = {
    "id": uuidutils.generate_uuid(dashed=True),
    "vpnserver_id": uuidutils.generate_uuid(dashed=True),
    "compute_id": uuidutils.generate_uuid(dashed=True),
    "lb_network_ip": "192.168.1.3",
    "vrrp_ip": "192.168.1.6",
    "ha_ip": "192.168.1.10",
    "vrrp_port_id": uuidutils.generate_uuid(dashed=True),
    "ha_port_id": uuidutils.generate_uuid(dashed=True),
    "cert_expiration": "2019-09-19 00:34:51",
    "cert_busy": 0,
    "role": "BACKUP",
    "status": "ALLOCATED",
    "vrrp_interface": "eth1",
    "vrrp_id": 1,
    "vrrp_priority": 200,
    "cached_zone": "zone2",
    "image_id": uuidutils.generate_uuid(dashed=True),
}

HM_ATTRS = {
    "project_id": uuidutils.generate_uuid(dashed=True),
    "name": "hm-name-" + uuidutils.generate_uuid(dashed=True),
    "admin_state_up": True,
    "pools": [
        {
            "id": uuidutils.generate_uuid(dashed=True)
        }
    ],
    "created_at": "2017-05-10T06:11:10",
    "provisioning_status": "PENDING_CREATE",
    "delay": 10,
    "expected_codes": "200",
    "max_retries": 2,
    "http_method": "GET",
    "timeout": 10,
    "max_retries_down": 3,
    "url_path": "/some/custom/path",
    "type": "HTTP",
    "id": uuidutils.generate_uuid(dashed=True),
    "http_version": 1.1,
    "domain_name": "testlab.com"
}

LISTENER_ATTRS = {
    "id": uuidutils.generate_uuid(),
    "name": "li-name-" + uuidutils.generate_uuid(dashed=True),
    "project_id": uuidutils.generate_uuid(dashed=True),
    "protocol": "HTTP",
    "protocol_port": 80,
    "provisioning_status": "ACTIVE",
    "default_pool_id": None,
    "vpnservers": None,
    "connection_limit": 10,
    "admin_state_up": True,
    "default_tls_container_ref": uuidutils.generate_uuid(dashed=True),
    "sni_container_refs": [uuidutils.generate_uuid(dashed=True),
                           uuidutils.generate_uuid(dashed=True)],
    "timeout_client_data": 50000,
    "timeout_member_connect": 5000,
    "timeout_member_data": 50000,
    "timeout_tcp_inspect": 0,
    'client_ca_tls_container_ref': uuidutils.generate_uuid(dashed=True),
    'client_authentication': "OPTIONAL",
    'client_crl_container_ref': uuidutils.generate_uuid(dashed=True),
    "allowed_cidrs": ['192.0.2.0/24', '198.51.100.0/24'],
    'tls_ciphers': "ECDHE-RSA-AES256-GCM-SHA384:ECDHE-RSA-AES128-GCM-SHA256",
    'tls_versions': ['TLSv1.1', 'TLSv1.2'],
    'alpn_protocols': ['h2', 'http/1.1']
}

VPNSERVER_ATTRS = {
    "id": uuidutils.generate_uuid(),
    "name": "lb-name-" + uuidutils.generate_uuid(dashed=True),
    "project_id": uuidutils.generate_uuid(dashed=True),
    "vip_address": "192.0.2.124",
    "vip_network_id": uuidutils.generate_uuid(dashed=True),
    "vip_subnet_id": uuidutils.generate_uuid(dashed=True),
    "vip_qos_policy_id": uuidutils.generate_uuid(dashed=True),
    "provisioning_status": "ACTIVE",
    "operating_status": "ONLINE",
    "provider": "tunnel",
    "flavor_id": uuidutils.generate_uuid(dashed=True),
}

L7POLICY_ATTRS = {
    "listener_id": uuidutils.generate_uuid(),
    "description": "fake desc",
    "admin_state_up": True,
    "rules": [{"id": uuidutils.generate_uuid()}],
    "provisioning_status": "active",
    "redirect_pool_id": uuidutils.generate_uuid(),
    "action": "POOL_REDIRECT",
    "position": 1,
    "project_id": uuidutils.generate_uuid(),
    "id": uuidutils.generate_uuid(),
    "name": "l7po-name-" + uuidutils.generate_uuid(dashed=True),
}

L7RULE_ATTRS = {
    "created_at": "2017-05-04T18:46:35",
    "compare_type": "ENDS_WITH",
    "provisioning_status": "ACTIVE",
    "invert": False,
    "admin_state_up": True,
    "value": ".example.com",
    "key": None,
    "project_id": uuidutils.generate_uuid(),
    "type": "HOST_NAME",
    "id": uuidutils.generate_uuid(),
    "operating_status": "ONLINE",
}

MEMBER_ATTRS = {
    "project_id": uuidutils.generate_uuid(dashed=True),
    "name": "test-member",
    "backup": False,
    "weight": 1,
    "admin_state_up": True,
    "subnet_id": uuidutils.generate_uuid(dashed=True),
    "tenant_id": uuidutils.generate_uuid(dashed=True),
    "provisioning_status": "ACTIVE",
    "address": "192.0.2.122",
    "protocol_port": 80,
    "id": uuidutils.generate_uuid(dashed=True),
    "operating_status": "NO_MONITOR",
    "pool_id": uuidutils.generate_uuid(dashed=True),
}

POOL_ATTRS = {
    "admin_state_up": True,
    "description": "fake desc",
    "id": uuidutils.generate_uuid(),
    "lb_algorithm": "ROUND_ROBIN",
    "listeners": [{"id": uuidutils.generate_uuid()}],
    "vpnservers": [{"id": uuidutils.generate_uuid()}],
    "members": [{"id": uuidutils.generate_uuid()}],
    "name": "po-name-" + uuidutils.generate_uuid(dashed=True),
    "project_id": uuidutils.generate_uuid(dashed=True),
    "protocol": "HTTP",
    "provisioning_status": "ACTIVE",
    "tls_container_ref": uuidutils.generate_uuid(),
    "ca_tls_container_ref": uuidutils.generate_uuid(),
    "crl_container_ref": uuidutils.generate_uuid(),
    "tls_enabled": True,
    "tls_ciphers": "ECDHE-RSA-AES256-GCM-SHA384:ECDHE-RSA-AES128-GCM-SHA256",
    "tls_versions": ['TLSv1.1', 'TLSv1.2']
}

QUOTA_ATTRS = {
    "health_monitor": -1,
    "listener": None,
    "vpn_server": 5,
    "member": 50,
    "pool": None,
    "l7policy": 20,
    "l7rule": 30,
    "project_id": uuidutils.generate_uuid(dashed=True),
}

PROVIDER_ATTRS = {
    "name": "provider1",
    "description": "Description of provider1."
}

CAPABILITY_ATTRS = {
    "name": "some_capability",
    "description": "Description of capability."
}

FLAVOR_ATTRS = {
    "id": uuidutils.generate_uuid(),
    "name": "fv-name-" + uuidutils.generate_uuid(dashed=True),
    "flavor_profile_id": None,
    "enabled": True,
}

FLAVORPROFILE_ATTRS = {
    "id": uuidutils.generate_uuid(),
    "name": "fvpf-name-" + uuidutils.generate_uuid(dashed=True),
    "provider_name": "mock_provider",
    "flavor_data": '{"mock_key": "mock_value"}',
}

AVAILABILITY_ZONE_ATTRS = {
    "name": "az-name-" + uuidutils.generate_uuid(dashed=True),
    "availability_zone_profile_id": None,
    "enabled": True,
    "description": "Description of AZ",
}

AVAILABILITY_ZONE_PROFILE_ATTRS = {
    "id": uuidutils.generate_uuid(),
    "name": "azpf-name-" + uuidutils.generate_uuid(dashed=True),
    "provider_name": "mock_provider",
    "availabilityzone_data": '{"mock_key": "mock_value"}',
}



#   Copyright 2019 Red Hat, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import copy
from unittest import mock

from osc_lib import exceptions

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import health_monitor
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestHealthMonitor(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._hm = fakes.createFakeResource('hm')
        self.hm_info = copy.deepcopy(attr_consts.HM_ATTRS)
        self.columns = copy.deepcopy(constants.MONITOR_COLUMNS)

        info_list = {'healthmonitors': [
            {k: v for k, v in attr_consts.HM_ATTRS.items() if (
                k in self.columns)},
        ]}
        self.api_mock = mock.Mock()
        self.api_mock.health_monitor_list.return_value = info_list
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestHealthMonitorList(TestHealthMonitor):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(attr_consts.HM_ATTRS[k] for k in self.columns),)
        self.cmd = health_monitor.ListHealthMonitor(self.app, None)

    def test_health_monitor_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.health_monitor_list.assert_called_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestHealthMonitorDelete(TestHealthMonitor):

    def setUp(self):
        super().setUp()
        self.cmd = health_monitor.DeleteHealthMonitor(self.app, None)

    def test_health_monitor_delete(self):
        arglist = [self._hm.id]
        verifylist = [
            ('health_monitor', self._hm.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_delete.assert_called_with(
            health_monitor_id=self._hm.id)

    @mock.patch('osc_lib.utils.wait_for_delete')
    def test_health_monitor_delete_wait(self, mock_wait):
        arglist = [self._hm.id, '--wait']
        verifylist = [
            ('health_monitor', self._hm.id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_delete.assert_called_with(
            health_monitor_id=self._hm.id)
        mock_wait.assert_called_once_with(
            manager=mock.ANY,
            res_id=self._hm.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_health_monitor_delete_failure(self):
        arglist = ['unknown_hm']
        verifylist = [
            ('health_monitor', 'unknown_hm')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)
        self.assertNotCalled(self.api_mock.health_monitor_delete)


class TestHealthMonitorCreate(TestHealthMonitor):

    def setUp(self):
        super().setUp()
        self.api_mock = mock.Mock()
        self.api_mock.health_monitor_create.return_value = {
            'healthmonitor': self.hm_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = health_monitor.CreateHealthMonitor(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_health_monitor_attrs')
    def test_health_monitor_create(self, mock_client):
        mock_client.return_value = self.hm_info
        arglist = ['mock_pool_id',
                   '--name', self._hm.name,
                   '--delay', str(self._hm.delay),
                   '--timeout', str(self._hm.timeout),
                   '--max-retries', str(self._hm.max_retries),
                   '--type', self._hm.type.lower(),
                   '--http-method', self._hm.http_method.lower(),
                   '--http-version', str(self._hm.http_version),
                   '--domain-name', self._hm.domain_name]
        verifylist = [
            ('pool', 'mock_pool_id'),
            ('name', self._hm.name),
            ('delay', str(self._hm.delay)),
            ('timeout', str(self._hm.timeout)),
            ('max_retries', self._hm.max_retries),
            ('type', self._hm.type),
            ('http_method', self._hm.http_method),
            ('http_version', self._hm.http_version),
            ('domain_name', self._hm.domain_name)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_create.assert_called_with(
            json={'healthmonitor': self.hm_info})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_health_monitor_attrs')
    def test_health_monitor_create_wait(self, mock_client, mock_wait):
        self.hm_info['pools'] = [{'id': 'mock_pool_id'}]
        mock_client.return_value = self.hm_info
        self.api_mock.pool_show.return_value = {
            'vpnservers': [{'id': 'mock_lb_id'}]}
        self.api_mock.health_monitor_show.return_value = self.hm_info
        arglist = ['mock_pool_id',
                   '--name', self._hm.name,
                   '--delay', str(self._hm.delay),
                   '--timeout', str(self._hm.timeout),
                   '--max-retries', str(self._hm.max_retries),
                   '--type', self._hm.type.lower(),
                   '--http-method', self._hm.http_method.lower(),
                   '--http-version', str(self._hm.http_version),
                   '--domain-name', self._hm.domain_name,
                   '--wait']
        verifylist = [
            ('pool', 'mock_pool_id'),
            ('name', self._hm.name),
            ('delay', str(self._hm.delay)),
            ('timeout', str(self._hm.timeout)),
            ('max_retries', self._hm.max_retries),
            ('type', self._hm.type),
            ('http_method', self._hm.http_method),
            ('http_version', self._hm.http_version),
            ('domain_name', self._hm.domain_name),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_create.assert_called_with(
            json={'healthmonitor': self.hm_info})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id='mock_lb_id',
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestHealthMonitorShow(TestHealthMonitor):

    def setUp(self):
        super().setUp()
        self.api_mock.health_monitor_show.return_value = {
            'healthmonitor': self.hm_info,
        }
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = health_monitor.ShowHealthMonitor(self.app, None)

    def test_health_monitor_show(self):
        arglist = [self._hm.id]
        verifylist = [
            ('health_monitor', self._hm.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_show.assert_called_with(
            health_monitor_id=self._hm.id)


class TestHealthMonitorSet(TestHealthMonitor):

    def setUp(self):
        super().setUp()
        self.cmd = health_monitor.SetHealthMonitor(self.app, None)

    def test_health_monitor_set(self):
        arglist = [self._hm.id, '--name', 'new_name',
                   '--http-version', str(self._hm.http_version),
                   '--domain-name', self._hm.domain_name]
        verifylist = [
            ('health_monitor', self._hm.id),
            ('name', 'new_name'),
            ('http_version', self._hm.http_version),
            ('domain_name', self._hm.domain_name)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_set.assert_called_with(
            self._hm.id, json={'healthmonitor': {
                'name': 'new_name', 'http_version': self._hm.http_version,
                'domain_name': self._hm.domain_name}})

    @mock.patch('osc_lib.utils.wait_for_status')
    def test_health_monitor_set_wait(self, mock_wait):
        arglist = [self._hm.id, '--name', 'new_name', '--wait']
        verifylist = [
            ('health_monitor', self._hm.id),
            ('name', 'new_name'),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_set.assert_called_with(
            self._hm.id, json={'healthmonitor': {'name': 'new_name'}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._hm.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestHealthMonitorUnset(TestHealthMonitor):
    PARAMETERS = ('name', 'domain_name', 'expected_codes', 'http_method',
                  'http_version', 'max_retries_down', 'url_path')

    def setUp(self):
        super().setUp()
        self.cmd = health_monitor.UnsetHealthMonitor(self.app, None)

    def test_hm_unset_domain_name(self):
        self._test_hm_unset_param('domain_name')

    def test_hm_unset_expected_codes(self):
        self._test_hm_unset_param('expected_codes')

    def test_hm_unset_http_method(self):
        self._test_hm_unset_param('http_method')

    def test_hm_unset_http_version(self):
        self._test_hm_unset_param('http_version')

    def test_hm_unset_max_retries_down(self):
        self._test_hm_unset_param('max_retries_down')

    def test_hm_unset_name(self):
        self._test_hm_unset_param('name')

    def test_hm_unset_name_wait(self):
        self._test_hm_unset_param_wait('name')

    def test_hm_unset_url_path(self):
        self._test_hm_unset_param('url_path')

    def _test_hm_unset_param(self, param):
        self.api_mock.health_monitor_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._hm.id, '--%s' % arg_param]
        ref_body = {'healthmonitor': {param: None}}
        verifylist = [
            ('health_monitor', self._hm.id),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_set.assert_called_once_with(
            self._hm.id, json=ref_body)

    @mock.patch('osc_lib.utils.wait_for_status')
    def _test_hm_unset_param_wait(self, param, mock_wait):
        self.api_mock.health_monitor_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._hm.id, '--%s' % arg_param, '--wait']
        ref_body = {'healthmonitor': {param: None}}
        verifylist = [
            ('health_monitor', self._hm.id),
            ('wait', True),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_set.assert_called_once_with(
            self._hm.id, json=ref_body)
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._hm.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_hm_unset_all(self):
        self.api_mock.health_monitor_set.reset_mock()
        ref_body = {'healthmonitor': {x: None for x in self.PARAMETERS}}
        arglist = [self._hm.id]
        for ref_param in self.PARAMETERS:
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('health_monitor', self._hm.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_set.assert_called_once_with(
            self._hm.id, json=ref_body)

    def test_hm_unset_none(self):
        self.api_mock.health_monitor_set.reset_mock()
        arglist = [self._hm.id]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('health_monitor', self._hm.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.health_monitor_set.assert_not_called()



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import copy
from unittest import mock

from osc_lib import exceptions

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import listener
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestListener(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._listener = fakes.createFakeResource('listener')
        self.listener_info = copy.deepcopy(attr_consts.LISTENER_ATTRS)
        self.columns = copy.deepcopy(constants.LISTENER_COLUMNS)

        self.api_mock = mock.Mock()
        self.api_mock.listener_list.return_value = copy.deepcopy(
            {'listeners': [attr_consts.LISTENER_ATTRS]})
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestListenerList(TestListener):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.LISTENER_ATTRS[k] for k in self.columns),)
        self.cmd = listener.ListListener(self.app, None)

    def test_listener_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.listener_list.assert_called_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    def test_listener_list_with_options(self):
        arglist = ['--name', 'rainbarrel']
        verifylist = [('name', 'rainbarrel')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.api_mock.listener_list.assert_called_with(name='rainbarrel')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestListenerDelete(TestListener):

    def setUp(self):
        super().setUp()
        self.cmd = listener.DeleteListener(self.app, None)

    def test_listener_delete(self):
        arglist = [self._listener.id]
        verifylist = [
            ('listener', self._listener.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_delete.assert_called_with(
            listener_id=self._listener.id)

    @mock.patch('osc_lib.utils.wait_for_delete')
    def test_listener_delete_wait(self, mock_wait):
        arglist = [self._listener.id, '--wait']
        verifylist = [
            ('listener', self._listener.id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_delete.assert_called_with(
            listener_id=self._listener.id)
        mock_wait.assert_called_once_with(
            manager=mock.ANY,
            res_id=self._listener.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_listener_delete_failure(self):
        arglist = ['unknown_lb']
        verifylist = [
            ('listener', 'unknown_lb')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)
        self.assertNotCalled(self.api_mock.listener_delete)


class TestListenerCreate(TestListener):

    def setUp(self):
        super().setUp()
        self.api_mock.listener_create.return_value = {
            'listener': self.listener_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = listener.CreateListener(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_listener_attrs')
    def test_listener_create(self, mock_client):
        mock_client.return_value = self.listener_info
        arglist = ['mock_lb_id',
                   '--name', self._listener.name,
                   '--protocol', 'HTTP',
                   '--protocol-port', '80']
        verifylist = [
            ('vpnserver', 'mock_lb_id'),
            ('name', self._listener.name),
            ('protocol', 'HTTP'),
            ('protocol_port', 80)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_create.assert_called_with(
            json={'listener': self.listener_info})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_listener_attrs')
    def test_listener_create_wait(self, mock_client, mock_wait):
        self.listener_info['vpnservers'] = [{'id': 'mock_lb_id'}]
        mock_client.return_value = self.listener_info
        self.api_mock.listener_show.return_value = self.listener_info
        arglist = ['mock_lb_id',
                   '--name', self._listener.name,
                   '--protocol', 'HTTP',
                   '--protocol-port', '80',
                   '--wait']
        verifylist = [
            ('vpnserver', 'mock_lb_id'),
            ('name', self._listener.name),
            ('protocol', 'HTTP'),
            ('protocol_port', 80),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_create.assert_called_with(
            json={'listener': self.listener_info})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self.listener_info['vpnservers'][0]['id'],
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    @mock.patch('tunnelclient.osc.v2.utils.get_listener_attrs')
    def test_tls_listener_create(self, mock_client):
        mock_client.return_value = self.listener_info
        arglist = ['mock_lb_id',
                   '--name', self._listener.name,
                   '--protocol', 'TERMINATED_HTTPS'.lower(),
                   '--protocol-port', '443',
                   '--sni-container-refs',
                   self._listener.sni_container_refs[0],
                   self._listener.sni_container_refs[1],
                   '--default-tls-container-ref',
                   self._listener.default_tls_container_ref,
                   '--client-ca-tls-container-ref',
                   self._listener.client_ca_tls_container_ref,
                   '--client-authentication',
                   self._listener.client_authentication,
                   '--client-crl-container-ref',
                   self._listener.client_crl_container_ref,
                   '--tls-ciphers',
                   self._listener.tls_ciphers,
                   '--tls-version',
                   self._listener.tls_versions[0],
                   '--tls-version',
                   self._listener.tls_versions[1],
                   '--alpn-protocol',
                   self._listener.alpn_protocols[0],
                   '--alpn-protocol',
                   self._listener.alpn_protocols[1]]

        verifylist = [
            ('vpnserver', 'mock_lb_id'),
            ('name', self._listener.name),
            ('protocol', 'TERMINATED_HTTPS'),
            ('protocol_port', 443),
            ('sni_container_refs', self._listener.sni_container_refs),
            ('default_tls_container_ref',
             self._listener.default_tls_container_ref),
            ('client_ca_tls_container_ref',
             self._listener.client_ca_tls_container_ref),
            ('client_authentication', self._listener.client_authentication),
            ('client_crl_container_ref',
             self._listener.client_crl_container_ref),
            ('tls_ciphers',
             self._listener.tls_ciphers),
            ('tls_versions',
             self._listener.tls_versions),
            ('alpn_protocols',
             self._listener.alpn_protocols),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_create.assert_called_with(
            json={'listener': self.listener_info})

    @mock.patch('tunnelclient.osc.v2.utils.get_listener_attrs')
    def test_listener_create_timeouts(self, mock_client):
        mock_client.return_value = self.listener_info
        arglist = ['mock_lb_id',
                   '--name', self._listener.name,
                   '--protocol', 'HTTP',
                   '--protocol-port', '80',
                   '--timeout-client-data', '123',
                   '--timeout-member-connect', '234',
                   '--timeout-member-data', '345',
                   '--timeout-tcp-inspect', '456']
        verifylist = [
            ('vpnserver', 'mock_lb_id'),
            ('name', self._listener.name),
            ('protocol', 'HTTP'),
            ('protocol_port', 80),
            ('timeout_client_data', 123),
            ('timeout_member_connect', 234),
            ('timeout_member_data', 345),
            ('timeout_tcp_inspect', 456),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_create.assert_called_with(
            json={'listener': self.listener_info})


class TestListenerShow(TestListener):

    def setUp(self):
        super().setUp()
        self.api_mock.listener_show.return_value = self.listener_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = listener.ShowListener(self.app, None)

    def test_listener_show(self):
        arglist = [self._listener.id]
        verifylist = [
            ('listener', self._listener.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_show.assert_called_with(
            listener_id=self._listener.id)


class TestListenerSet(TestListener):

    def setUp(self):
        super().setUp()
        self.cmd = listener.SetListener(self.app, None)

    def test_listener_set(self):
        arglist = [self._listener.id, '--name', 'new_name',
                   '--sni-container-refs',
                   self._listener.sni_container_refs[0],
                   self._listener.sni_container_refs[1],
                   '--default-tls-container-ref',
                   self._listener.default_tls_container_ref,
                   '--client-ca-tls-container-ref',
                   self._listener.client_ca_tls_container_ref,
                   '--client-authentication',
                   self._listener.client_authentication,
                   '--client-crl-container-ref',
                   self._listener.client_crl_container_ref,
                   '--allowed-cidr',
                   self._listener.allowed_cidrs[0],
                   '--allowed-cidr',
                   self._listener.allowed_cidrs[1],
                   '--tls-ciphers',
                   self._listener.tls_ciphers,
                   '--tls-version',
                   self._listener.tls_versions[0],
                   '--tls-version',
                   self._listener.tls_versions[1],
                   '--alpn-protocol',
                   self._listener.alpn_protocols[0],
                   '--alpn-protocol',
                   self._listener.alpn_protocols[1]]
        verifylist = [
            ('listener', self._listener.id),
            ('name', 'new_name'),
            ('sni_container_refs', self._listener.sni_container_refs),
            ('default_tls_container_ref',
                self._listener.default_tls_container_ref),
            ('client_ca_tls_container_ref',
             self._listener.client_ca_tls_container_ref),
            ('client_authentication',
             self._listener.client_authentication),
            ('client_crl_container_ref',
             self._listener.client_crl_container_ref),
            ('allowed_cidrs', self._listener.allowed_cidrs),
            ('tls_ciphers', self._listener.tls_ciphers),
            ('tls_versions', self._listener.tls_versions),
            ('alpn_protocols', self._listener.alpn_protocols)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_set.assert_called_with(
            self._listener.id, json={
                'listener': {
                    'name': 'new_name',
                    'sni_container_refs': self._listener.sni_container_refs,
                    'default_tls_container_ref':
                        self._listener.default_tls_container_ref,
                    'client_ca_tls_container_ref':
                        self._listener.client_ca_tls_container_ref,
                    'client_authentication':
                        self._listener.client_authentication,
                    'client_crl_container_ref':
                        self._listener.client_crl_container_ref,
                    'allowed_cidrs': self._listener.allowed_cidrs,
                    'tls_ciphers': self._listener.tls_ciphers,
                    'tls_versions': self._listener.tls_versions,
                    'alpn_protocols': self._listener.alpn_protocols,
                }})

    @mock.patch('osc_lib.utils.wait_for_status')
    def test_listener_set_wait(self, mock_wait):
        arglist = [self._listener.id, '--name', 'new_name', '--wait']
        verifylist = [
            ('listener', self._listener.id),
            ('name', 'new_name'),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_set.assert_called_with(
            self._listener.id, json={'listener': {'name': 'new_name'}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._listener.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestListenerStatsShow(TestListener):

    def setUp(self):
        super().setUp()
        listener_stats_info = {'stats': {'bytes_in': '0'}}
        self.api_mock.listener_stats_show.return_value = {
            'stats': listener_stats_info['stats']}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock
        self.cmd = listener.ShowListenerStats(self.app, None)

    def test_listener_stats_show(self):
        arglist = [self._listener.id]
        verifylist = [
            ('listener', self._listener.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_stats_show.assert_called_with(
            listener_id=self._listener.id)


class TestListenerUnset(TestListener):
    PARAMETERS = ('name', 'description', 'connection_limit', 'default_pool_id',
                  'default_tls_container_ref', 'sni_container_refs',
                  'insert_headers', 'timeout_client_data',
                  'timeout_member_connect', 'timeout_member_data',
                  'timeout_tcp_inspect', 'client_ca_tls_container_ref',
                  'client_authentication', 'client_crl_container_ref',
                  'allowed_cidrs', 'tls_versions', 'tls_ciphers')

    def setUp(self):
        super().setUp()
        self.cmd = listener.UnsetListener(self.app, None)

    def test_listener_unset_name(self):
        self._test_listener_unset_param('name')

    def test_listener_unset_name_wait(self):
        self._test_listener_unset_param_wait('name')

    def test_listener_unset_description(self):
        self._test_listener_unset_param('description')

    def test_listener_unset_connection_limit(self):
        self._test_listener_unset_param('connection_limit')

    def test_listener_unset_default_pool(self):
        self._test_listener_unset_param('default_pool')

    def test_listener_unset_default_tls_container_ref(self):
        self._test_listener_unset_param('default_tls_container_ref')

    def test_listener_unset_sni_container_refs(self):
        self._test_listener_unset_param('sni_container_refs')

    def test_listener_unset_insert_headers(self):
        self._test_listener_unset_param('insert_headers')

    def test_listener_unset_timeout_client_data(self):
        self._test_listener_unset_param('timeout_client_data')

    def test_listener_unset_timeout_member_connect(self):
        self._test_listener_unset_param('timeout_member_connect')

    def test_listener_unset_timeout_member_data(self):
        self._test_listener_unset_param('timeout_member_data')

    def test_listener_unset_timeout_tcp_inspect(self):
        self._test_listener_unset_param('timeout_tcp_inspect')

    def test_listener_unset_client_ca_tls_container_ref(self):
        self._test_listener_unset_param('client_ca_tls_container_ref')

    def test_listener_unset_client_authentication(self):
        self._test_listener_unset_param('client_authentication')

    def test_listener_unset_client_crl_container_ref(self):
        self._test_listener_unset_param('client_crl_container_ref')

    def test_listener_unset_allowed_cidrs(self):
        self._test_listener_unset_param('allowed_cidrs')

    def test_listener_unset_tls_versions(self):
        self._test_listener_unset_param('tls_versions')

    def test_listener_unset_tls_ciphers(self):
        self._test_listener_unset_param('tls_ciphers')

    def _test_listener_unset_param(self, param):
        self.api_mock.listener_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._listener.id, '--%s' % arg_param]
        # Handle the special rename case of default_pool rename
        if param == 'default_pool':
            param = 'default_pool_id'
        ref_body = {'listener': {param: None}}
        verifylist = [
            ('listener', self._listener.id),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_set.assert_called_once_with(
            self._listener.id, json=ref_body)

    @mock.patch('osc_lib.utils.wait_for_status')
    def _test_listener_unset_param_wait(self, param, mock_wait):
        self.api_mock.listener_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._listener.id, '--%s' % arg_param, '--wait']
        # Handle the special rename case of default_pool rename
        if param == 'default_pool':
            param = 'default_pool_id'
        ref_body = {'listener': {param: None}}
        verifylist = [
            ('listener', self._listener.id),
            ('wait', True),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_set.assert_called_once_with(
            self._listener.id, json=ref_body)
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._listener.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_listener_unset_all(self):
        self.api_mock.listener_set.reset_mock()
        ref_body = {'listener': {x: None for x in self.PARAMETERS}}
        arglist = [self._listener.id]
        for ref_param in self.PARAMETERS:
            # Handle the special rename case of default_pool rename
            if ref_param == 'default_pool_id':
                ref_param = 'default_pool'
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('listener', self._listener.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_set.assert_called_once_with(
            self._listener.id, json=ref_body)

    def test_listener_unset_none(self):
        self.api_mock.listener_set.reset_mock()
        arglist = [self._listener.id]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('listener', self._listener.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.listener_set.assert_not_called()
